
#include "unix.h"
#include "off.h"
#include "xpex.h"
#include "global.h"

char *FileName = (char *)NULL;
OFFProperty *newprop;

#define ZERO_TOLERANCE 1.0e-30
#define ABS(_x) ((_x) < 0.0 ? -(_x) : (_x))
#define NEAR_ZERO(_s) (ABS (_s) < ZERO_TOLERANCE)

#define CROSS_PRODUCT(_p1, _p2, _p3, _p4, _v)                   \
{                                                               \
    (_v)->x = (((_p2)->y - (_p1)->y) * ((_p4)->z - (_p3)->z)) - \
              (((_p2)->z - (_p1)->z) * ((_p4)->y - (_p3)->y));  \
    (_v)->y = (((_p2)->z - (_p1)->z) * ((_p4)->x - (_p3)->x)) - \
              (((_p2)->x - (_p1)->x) * ((_p4)->z - (_p3)->z));  \
    (_v)->z = (((_p2)->x - (_p1)->x) * ((_p4)->y - (_p3)->y)) - \
              (((_p2)->y - (_p1)->y) * ((_p4)->x - (_p3)->x));  \
}

#define INVERT_VECTOR(_v)          \
{                                  \
    (_v)->x *= -1.;                \
    (_v)->y *= -1.;                \
    (_v)->z *= -1.;                \
}

#define DOT_PRODUCT(_v1, _v2)      \
    ((_v1)->x * (_v2)->x + (_v1)->y * (_v2)->y + (_v1)->z * (_v2)->z)

#define NORMALIZE_VECTOR(_v, _len) \
{                                  \
    _len = DOT_PRODUCT (_v, _v);   \
    _len = sqrt (_len);            \
    if (!NEAR_ZERO (_len))         \
    {                              \
        (_v)->x /= _len;           \
        (_v)->y /= _len;           \
        (_v)->z /= _len;           \
    }                              \
}


getNormals( path, file )
char *path;
char *file;
{
    char *off_aofffile = (char *)NULL;
    char *off_geomfile = (char *)NULL;
    char *off_polyNormalsfile = (char *)NULL;
    char *off_vertNormalsfile = (char *)NULL;

    char *temp, *rc;

    struct stat s_buf;

    char  *ptr;
    long  *lptr;
    float *fptr;
    short *sptr;

    long nver, nfac, nedges;
    unsigned int nvertices, nfacets;

    Vertex *vertices;
    Facet  *facets;

    Vector *vertex_normals;
    Vector *facet_normals;

    FILE *fp, *fvn, *fpn;

    register int  i, j, k;
    int  wantPnorm = 1, wantVnorm = 1;

    if( (off_aofffile = (char *)malloc( strlen( path ) + strlen( file ) + 5 + 1 )) == (char *)0 ) {
        fprintf( stderr, "Cant allocate char\n" );
        exit(1);
    }
    strcpy( off_aofffile, path );
    strcat( off_aofffile, file );
    strcat( off_aofffile, ".aoff" );

    if( stat( off_aofffile, &s_buf ) == -1 ) {
        fprintf( stderr, "File \"%s\" does not exist 3\n", off_aofffile );
        exit(-1);
    }
    if( (fp = fopen( off_aofffile, "r" )) == (FILE *)NULL ) {
        fprintf( stderr, "Cannot open file %s for readin\n", off_aofffile );
        exit(-1);
    }
    if( (temp = (char *)malloc( 80 )) == (char *)0 ) {
        fprintf( stderr, "Cant allocate char\n" );
        exit(1);
    }
    while( (rc = fgets( temp, 80, fp )) != (char *)NULL ) {
        char *rc;
        if( (rc = strstr( temp, ".pnorm" )) != (char *)NULL ) wantPnorm = 0;
        if( (rc = strstr( temp, ".vnorm" )) != (char *)NULL ) wantVnorm = 0;
    }
    fclose(fp);

    if( (off_polyNormalsfile = (char *)malloc(strlen(path)+strlen(file)+strlen(".pnorm")+1)) == (char *)0 ) {
        fprintf( stderr, "Cant allocate char\n" );
        exit(1);
    }
    strcat( strcat( strcpy( off_polyNormalsfile, path ), file ), ".pnorm" );
    if( wantPnorm ) {
        FILE *fa;
        if( (fa = fopen( off_aofffile, "a" )) == (FILE *)NULL ) {
            fprintf( stderr, "Cannot open file %s for apending\n",off_aofffile);
            exit(-1);
        }
        fprintf( fa, "polygon_normals\t\tgeneric\t\tfff\t\t%s.pnorm\n", file );
        fclose(fa);
    }

    if( (off_vertNormalsfile = (char *)malloc(strlen(path)+strlen(file)+strlen(".vnorm")+1)) == (char *)0 ) {
        fprintf( stderr, "Cant allocate char\n" );
        exit(1);
    }
    strcat( strcat( strcpy( off_vertNormalsfile, path ), file ), ".vnorm" );
    if( wantVnorm ) {
        FILE *fa;
        if( (fa = fopen( off_aofffile, "a" )) == (FILE *)NULL ) {
            fprintf( stderr, "Cannot open file %s for apending\n",off_aofffile);
            exit(-1);
        }
        fprintf( fa, "vertex_normals\t\tgeneric\t\tfff\t\t%s.vnorm\n", file );
        fclose(fa);
    }

    if( !(stat( off_vertNormalsfile, &s_buf ) == -1) &&
        !(stat( off_polyNormalsfile, &s_buf ) == -1) ) {
        fprintf( stderr, "Warning: File \"%s\" exists\n", off_polyNormalsfile );
        fprintf( stderr, "Warning: File \"%s\" exists\n", off_vertNormalsfile );
    }

    if( (off_geomfile = (char *)malloc( strlen( path ) + strlen( file ) + 6 )) == (char *)0 ) {
        fprintf( stderr, "Cant allocate char\n" );
        exit(1);
    }
    strcat( strcat( strcpy( off_geomfile, path ), file ), ".geom" );
    if( stat( off_geomfile, &s_buf ) == -1 ) {
        if( (off_geomfile = (char *)malloc( strlen( file )+6 )) == (char *)0 ) {
            fprintf( stderr, "Cant allocate char\n" );
            exit(1);
        }
        strcat( strcat( strcpy( off_geomfile, path ), file ), ".bgeom" );
        if( stat( off_geomfile, &s_buf ) == -1 ) {
            fprintf( stderr, "File \"%s\" does not exist 4\n", off_geomfile );
            exit(-1);
        }
    }

    if( (newprop = (OFFProperty *)malloc(sizeof(OFFProperty))) == (OFFProperty *)0 ) {
        fprintf( stderr, "Cant allocate OFFProperty\n" );
        exit(1);
    }
    newprop->PropType = OFF_INDEXED_POLY_DATA;
    newprop->NextProp = (struct _OFFProp *)NULL;
    newprop->PropData = (char *)NULL;
    newprop->PropFileName[0] = '\0';
    newprop->DataFormat[0] = 'f';
    newprop->DataFormat[1] = 'f';
    newprop->DataFormat[2] = 'f';
    newprop->DataFormat[3] = '\0';
    newprop->PropCount = 0;
    newprop->PropName[0] = '\0';
    if( OFFReadIndexedPoly(newprop, off_geomfile) != 0 ) {
        fprintf( stderr, "Could not read file  \"%s\" \n", off_geomfile );
        exit(-1);
    }

    ptr = newprop->PropData;

    lptr = (long *) ptr;
    nver = *lptr++;
    nfac = *lptr++;
    nedges = *lptr++;
    ptr = (char *) lptr;

    nvertices = nver;
    nfacets   = nfac;

    if( (vertices = (Vertex *)malloc( nvertices * sizeof( Vertex ) )) == (Vertex *)0 ) {
        fprintf( stderr, "Cant allocate Vertex\n" );
        exit(1);
    }
    if( (facets   = (Facet *)malloc( nfacets * sizeof( Facet ) )) == (Facet *)0 ) {
        fprintf( stderr, "Cant allocate Facet\n" );
        exit(1);
    }

    fptr = (float *) ptr;
    for(i = 0; i < nvertices; i++) {
        vertices[i].x = *fptr++;
        vertices[i].y = *fptr++;
        vertices[i].z = *fptr++;
    }
    ptr = (char *) fptr;

    sptr = (short *) ptr;
    for(j = 0; j < nfacets; j++) {
        facets[j].count = *sptr++;
        if( (facets[j].edges = (Edge *)malloc( facets[j].count * sizeof( Edge ) )) == (Edge *)0 ) {
            fprintf( stderr, "Cant allocate Edge\n" );
            exit(1);
        }
    }
    for(j = 0; j < nfacets; j++)
        for(i = 0; i < facets[j].count; i++)
            facets[j].edges[i].e = *sptr++;
    ptr = (char *) sptr;

    if( (facet_normals = (Vector *)malloc( nfacets * sizeof( Vector ) )) == (Vector *)0 ) {
        fprintf( stderr, "Cant allocate Vector\n" );
        exit(1);
    }
    for(j = 0; j < nfacets; j++) {
        Vertex *v1, *v2, *v3;
        Vector *normal;
        int     found_v2, found_v3;
        float   dx, dy, dz;
        double  length;
        int     count = facets[j].count;
        int     countf = 0;

        v1 = (Vertex *) &vertices[(facets[j].edges[countf++].e)-1];
        count--;

        found_v2 = 0;
        while (!found_v2 && count > 0) {
            /* find v2, such that v2 is not coincident with v1 */

            v2 = (Vertex *) &vertices[(facets[j].edges[countf++].e)-1];
            count--;

            dx = v2->x - v1->x;
            dy = v2->y - v1->y;
            dz = v2->z - v1->z;

            if (!NEAR_ZERO (dx) || !NEAR_ZERO (dy) || !NEAR_ZERO (dz))
                found_v2 = 1;
        }

        found_v3 = 0;
        while (!found_v3 && count > 0) {
            /* find v3, such that v3 is non-colinear with v1 and v2 */
        
            v3 = (Vertex *) &vertices[(facets[j].edges[countf++].e)-1];
            count--;
        
            if( (normal = (Vector *)malloc( sizeof( Vector ) )) == (Vector *)0 ) {
                fprintf( stderr, "Cant allocate Vector\n" );
                exit(1);
            }
            CROSS_PRODUCT (v1, v2, v1, v3, normal);
            INVERT_VECTOR (normal);
            NORMALIZE_VECTOR (normal, length);
            if (!NEAR_ZERO (length))
                found_v3 = 1;
        }

        if (found_v3) {
            facet_normals[j].x = normal->x;
            facet_normals[j].y = normal->y;
            facet_normals[j].z = normal->z;
        }
#ifdef DEBUG
        else
            fprintf( stderr, "length is zero\n" );
#endif		/*  */
    }

    if( ( fpn = fopen( off_polyNormalsfile, "w" ) ) == (FILE *)NULL ) {
        fprintf( stderr, "Cannot open file %s\n", off_polyNormalsfile );
        exit(-1);
    }
    fprintf( fpn, "%d\n", nfacets );
    for( j = 0; j < nfacets; j++ )
        fprintf( fpn, "%f\t%f\t%f\n", facet_normals[j].x,
                 facet_normals[j].y, facet_normals[j].z );
    fclose(fpn);

    if( (vertex_normals = (Vector *)malloc( nvertices * sizeof( Vector ) )) == (Vector *)0 ) {
        fprintf( stderr, "Cant allocate Vector\n" );
        exit(1);
    }
    for( k = 0; k < nvertices; k++ ) {
        Vector *normal;
        double  length;

        if( (normal = (Vector *)malloc( sizeof( Vector ) )) == (Vector *)0 ) {
            fprintf( stderr, "Cant allocate Vector\n" );
            exit(1);
        }
        normal->x = normal->y = normal->z = 0.;
        for( j = 0; j < nfacets; j++ ) {
            for( i = 0; i < facets[j].count; i++ )
                if( facets[j].edges[i].e == k+1 ) {
                    normal->x += facet_normals[j].x;
                    normal->y += facet_normals[j].y;
                    normal->z += facet_normals[j].z;
                }
        }
        NORMALIZE_VECTOR( normal, length );
        INVERT_VECTOR( normal );
        if( !NEAR_ZERO (length) ) {
            vertex_normals[k].x = normal->x;
            vertex_normals[k].y = normal->y;
            vertex_normals[k].z = normal->z;
        }
#ifdef DEBUG
        else
            fprintf( stderr, "length is zero\n" );
#endif		/*  */
    }

    if( ( fvn = fopen( off_vertNormalsfile, "w" ) ) == (FILE *)NULL ) {
        fprintf( stderr, "Cannot open file %s\n", off_vertNormalsfile );
        exit(-1);
    }
    fprintf( fvn, "%d\n", nvertices );
    for( j = 0; j < nvertices; j++ )
        fprintf( fvn, "%f\t%f\t%f\n", vertex_normals[j].x,
                vertex_normals[j].y, vertex_normals[j].z );
    fclose( fvn );
}
